#include <metal_stdlib>
#include <simd/simd.h>

#import "VexilShaderTypes.h"

using namespace metal;

typedef struct
{
    packed_float3 position;
	packed_float3 normal;
	packed_float2 uv;
    float bone;
    float bone2;
    float weight;
} vertex_t;
typedef struct
{
	float4 bone0;
	float4 bone1;
	float4 bone2;
	float4 bone3;
} bones;
typedef struct
{
    float4 position [[position]];
	float2 texCoord;
	float3 transformedNormal;
} ColorInOut;

vertex ColorInOut vertexShader(const device vertex_t* vertex_array [[ buffer(0) ]],
							   const device bones* bone_array [[ buffer(1) ]],
                               constant Uniforms & uniforms [[ buffer(2) ]],
							   uint vid [[ vertex_id ]])
{
    ColorInOut out;

    float4 position = float4(vertex_array[vid].position, 1.0);
    int boneId = int(vertex_array[vid].bone);
    if ( boneId >= 0 )
    {
		float4 bone00 = bone_array[boneId].bone0;
		float4 bone01 = bone_array[boneId].bone1;
		float4 bone02 = bone_array[boneId].bone2;
		float4 bone03 = bone_array[boneId].bone3;
        int boneId2 = int(vertex_array[vid].bone2);
        float4x4 bone;
        if ( boneId2 < 0 )
        {
            bone = float4x4(bone00,bone01,bone02,bone03);
        } else {
            float weight = vertex_array[vid].weight;
            bone = float4x4(bone00,bone01,bone02,bone03);
            bone *= weight;
            float4 bone10 = bone_array[boneId2].bone0;
            float4 bone11 = bone_array[boneId2].bone1;
            float4 bone12 = bone_array[boneId2].bone2;
            float4 bone13 = bone_array[boneId2].bone3;
            float4x4 bone2 = float4x4(bone10,bone11,bone12,bone13);
            bone += bone2 * (1-weight);
        }
		out.position = uniforms.projectionMatrix * uniforms.modelViewMatrix * bone * position;
		float3x3 bone3x3 = float3x3(bone[0].xyz,bone[1].xyz,bone[2].xyz);
		out.transformedNormal = matrix_float3x3(uniforms.normalMatrix) * bone3x3 * vector_float3(vertex_array[vid].normal);
	}
    else
	{
		out.position = uniforms.projectionMatrix * uniforms.modelViewMatrix * position;
		out.transformedNormal = matrix_float3x3(uniforms.normalMatrix) * vector_float3(vertex_array[vid].normal);
	}
	out.texCoord = vertex_array[vid].uv;
	
    return out;
}

fragment float4 fragmentShader(ColorInOut in [[stage_in]],
							   texture2d<half> texture [[ texture(0) ]],
							   sampler texSampler [[sampler(0)]],
							   constant float4& diffuse [[buffer(0)]])
{
    float3 lightDirection = float3(1,1,-2);
    lightDirection = normalize(lightDirection);
	float3 vNormal = normalize(in.transformedNormal);
    float lightWeighting = clamp(dot(vNormal,lightDirection),0.0,1.0);
    if ( diffuse.x < 0 ) {
        float4 tex = float4(texture.sample(texSampler, in.texCoord.xy));
        return float4(tex.rgb*lightWeighting,tex.a);
    }
    else
	{
        return float4(diffuse.rgb*lightWeighting,diffuse.a);
	}
}
