import Foundation
import Metal
import MetalKit
import AVFoundation

class VexilMain : Renderer {
	
    var _model : VexilModel!
    var _model2 : VexilModel!
    var _grass : VexilModel!
    var _touch: Bool = false
    var _audioPlayer: AVAudioPlayer!
    var _audioGo: URL!
    var _audioCatch: URL!
    var _count:Int = -1

    override init?(metalKitView: MTKView) {
        super.init(metalKitView:metalKitView)
        _bgColor = MTLClearColor(red: 0.3, green: 0.6, blue: 1.0, alpha: 1)
        _model = loadModel(model: LoadModel(device:_device,fileName:"CatModel")!)
        _model2 = loadModel(model: LoadModel(device:_device,fileName:"DogModel")!)
        _grass = loadModel(model: LoadModel(device:_device,fileName:"GrassModel")!)
        _audioGo = URL(fileURLWithPath: Bundle.main.path(forResource: "Go", ofType: "wav")!)
        _audioCatch = URL(fileURLWithPath: Bundle.main.path(forResource: "Catch", ofType: "wav")!)
        initialize()
    }
    
    func initialize()
    {
        _count = -1
        _model._translation = simd_float3(750,500,750)
        _model2._translation = simd_float3(-500,500,-500)
        playAudio(url:_audioGo)
    }
    
    func playAudio(url:URL)
    {
        do {
            _audioPlayer = try AVAudioPlayer(contentsOf: url)
            _audioPlayer.play()
        } catch {
            print("AVAudioPlayer error")
        }
    }
    
    func modelMove()
    {
        var x:Float = _model._translation.x
        var y:Float = _model._translation.y + 1000
        var z:Float = _model._translation.z
        if _touch {
            let radians:Float = Transforms.radians_from_degrees(_model._rotation.y)
            x += sin(radians) * 4
            z -= cos(radians) * 4
        } else {
            _model._rotation.y += 2
            if _model._rotation.y >= 360 {
                _model._rotation.y -= 360
            }
        }
        let origin:simd_float3 = simd_float3(x,y,z)
        if ( _grass.intersect(origin: origin, dir: simd_float3(0,-1,0)) ) {
            y = origin.y - _grass._distance
            _model._translation = simd_float3(x,y,z)
        } else {
            _model._rotation.y += 180
        }
    }

    func getRandom(min:Float, max:Float)->Float {
        
        return ( Float(arc4random_uniform(UINT32_MAX)) / Float(UINT32_MAX) ) * (max - min) + min
    }

    func model2Move()
    {
        let radians:Float = Transforms.radians_from_degrees(_model2._rotation.y)
        let x:Float = _model2._translation.x + sin(radians) * 4
        var y:Float = _model2._translation.y + 100
        let z:Float = _model2._translation.z - cos(radians) * 4
        let origin:simd_float3 = simd_float3(x,y,z)
        if ( _grass.intersect(origin: origin, dir: simd_float3(0,-1,0)) ) {
            y = origin.y - _grass._distance
            _model2._translation = simd_float3(x,y,z)
        } else {
            _model2._rotation.y += 180
        }
        _model2._rotation.y += getRandom(min:-10,max:10)
        if _model2._rotation.y < 0 {
            _model2._rotation.y += 360
        }
        if _model2._rotation.y >= 360 {
            _model2._rotation.y -= 360
        }
    }
    
    func cameraMove()
    {
        let pos = _model._translation
        _eye = pos + simd_float3(500,500,500)
        _center = pos + simd_float3(0,200,0)
        _up = simd_float3(0,1,0)
    }

    func collide()
    {
        if _count < 0 {
            let x = _model._translation.x - _model2._translation.x
            let z = _model._translation.z - _model2._translation.z
            if sqrt(x*x + z*z) < 100 {
                _count = 0
                playAudio(url:_audioCatch)
            }
        } else {
            _count += 1
            if _count > 60*5 {
                initialize()
            }
        }
    }
    
    override func draw(in view: MTKView) {
        modelMove()
        model2Move()
        cameraMove()
        collide()

        super.draw(in: view)
    }

	func touchesBegan(_ touches: Set<UITouch>, with event: UIEvent?)  {
        _touch = true
	}

	func touchesMoved(_ touches: Set<UITouch>, with event: UIEvent?)  {
	}
	
	func touchesEnded(_ touches: Set<UITouch>, with event: UIEvent?)  {
		_touch = false
	}
}
